#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include "msditx.h"
#include "nortlib.h"
/* STILL Needed:
   library functions for hms(T) and myd(T)
*/

#define hms(x) "00:00:00"
#define mdy(x) "01-01-1996"
#define MAX_AUX_STR 200
#define MAXMSDCHANS  4

static char itx_aux_str[MAX_AUX_STR];
static int itx_aux_len;
static int itx_fno, itx_injno;

static FILE *itx_file0 = NULL;
static FILE *itx_file1 = NULL;
static FILE *itx_file2 = NULL;
static FILE *itx_file3 = NULL;
static FILE *itx_file4 = NULL;
static FILE *itx_file5 = NULL;
static FILE *itx_file6 = NULL;

void itx_end_inj( int chan ) {
  FILE *itx_file = SelectITX( chan );
  int injno;

  if ( chan == 0 )
	injno = itx_injno;
  else
	injno = itx_injno-1;

  if ( itx_file != NULL ) {
	/* write the end of the last injection, together with the auxiliary data string */
	fprintf( itx_file, "END\n" );
    fprintf( itx_file, "X note msdT%d_%05d, \" 1; %s \"\n", chan, injno, itx_aux_str );
	fprintf( itx_file, "X note msdR%d_%05d, \" 2; %s \"\n", chan, injno, itx_aux_str );
  }

}

void itx_new_inj( int fno, int injno, int chan ) {
  FILE *itx_file = SelectITX( chan );

  if ( fno != itx_fno ) {
	itx_end_inj( chan );
	fclose( itx_file );
	itx_file = NULL;
  }

  if ( itx_file == NULL ) {
	char fname[80];
	sprintf( fname, "msd%04d_%d.itx", fno, chan );
	if ((itx_file = fopen( fname, "w" )) == NULL)
		printf ("=====Cannot open %s\n", fname);	
	if ( itx_file == NULL )
	  nl_error( 3, "Unable to open output file %s", fname );
	fprintf( itx_file, "IGOR\n" );
  }

  fprintf( itx_file, "WAVES " );
  fprintf( itx_file, "msdT%d_%05d, msdR%d_%05d", chan, injno, chan, injno );
  fprintf( itx_file, "\nBEGIN\n" );
  itx_injno = injno;

  // set the file pointer.
  switch ( chan ) {
	case 0: itx_file0 = itx_file; break;
	case 1: itx_file1 = itx_file; break;
	case 2: itx_file2 = itx_file; break;
	case 3: itx_file3 = itx_file; break;
	case 4: itx_file4 = itx_file; break;
	case 5: itx_file5 = itx_file; break;
	case 6: itx_file6 = itx_file; break;
  }

}

/* write full name */
void itx_new_inj_flnm( int fno, int injno, int chan, char *hd, long int T ) {
  FILE *itx_file = SelectITX( chan );
  char  *path = "chroms";
  struct tm *tmbuf;
  struct stat buf;

  if ( fno != itx_fno ) {
	itx_end_inj( chan );
	fclose( itx_file );
	itx_file = NULL;
  }

  if ( itx_file == NULL ) {
	char fname[80];

	tmbuf = localtime( &T );

	/* make path dir if needed */
	if ( stat( path, &buf ) != 0 ) {
		mkdir ( path, S_IRWXU | S_IRWXG | S_IREAD );
	}

	sprintf( fname, "%s/%s%04d%02d%02d_%02d%02d%02d_%d.itx", path, hd, 
		tmbuf->tm_year + 1900, tmbuf->tm_mon+1, tmbuf->tm_mday,
		tmbuf->tm_hour, tmbuf->tm_min, tmbuf->tm_sec, chan );

	if ((itx_file = fopen( fname, "w" )) == NULL)
		printf ("=====Cannot open %s\n", fname);	
	if ( itx_file == NULL )
	  nl_error( 3, "Unable to open output file %s", fname );
	fprintf( itx_file, "IGOR\n" );
  }

  fprintf( itx_file, "WAVES " );
  fprintf( itx_file, "msdT%d_%05d, msdR%d_%05d", chan, injno, chan, injno );
  fprintf( itx_file, "\nBEGIN\n" );
  itx_injno = injno;

  // set the file pointer.
  switch ( chan ) {
	case 0: itx_file0 = itx_file; break;
	case 1: itx_file1 = itx_file; break;
	case 2: itx_file2 = itx_file; break;
	case 3: itx_file3 = itx_file; break;
	case 4: itx_file4 = itx_file; break;
	case 5: itx_file5 = itx_file; break;
	case 6: itx_file6 = itx_file; break;
  }

}


/* append additional string(s) to the itx_aux_string */
void itx_aux_data( char *t1, char *t2 ) {
  int l1, l2;
  l1 = strlen(t1);
  l2 = strlen(t2);
  assert( itx_aux_len + l1 + l2 < MAX_AUX_STR-1 );
  strcpy( itx_aux_str + itx_aux_len, t1 );
  strcpy( itx_aux_str + itx_aux_len + l1, t2 );
  itx_aux_len += l1 + l2;
}

/* initialize the itx_aux_string */
void itx_aux_data_init( unsigned long Sec, long int T, char *SP ) {
  struct tm *tmbuf;
  char tstr[12];

  sprintf( itx_aux_str, "%ld; ", Sec );
  itx_aux_len = strlen( itx_aux_str );
  tmbuf = localtime( &T );
  sprintf( tstr, "%02d:%02d:%02d", tmbuf->tm_hour, tmbuf->tm_min, tmbuf->tm_sec );
  itx_aux_data( tstr, "; " );
  sprintf( tstr, "%02d-%02d-%04d", tmbuf->tm_mon+1, tmbuf->tm_mday, tmbuf->tm_year + 1900 );
  itx_aux_data( tstr, "; " );
  itx_aux_data( SP, "; " );
}

void itx_ctr_dataL( long ctr, int chan ) {

	FILE *itx_file = SelectITX( chan );

	if ( itx_file == NULL )
		return;
	fprintf( itx_file, " %ld", ctr);
}

void itx_text_data( char *text, int chan ) {

	FILE *itx_file = SelectITX( chan );

	if ( itx_file == NULL )
		return;
	fprintf( itx_file, "%s", text );
}

FILE *SelectITX( chan ) {

	switch ( chan ) {
		case 0: return itx_file0;
		case 1: return itx_file1;
		case 2: return itx_file2;
		case 3: return itx_file3;
		case 4: return itx_file4;
		case 5: return itx_file5;
		case 6: return itx_file6;
	}

	return NULL;

}